import { ChainOrRpcUrl, SmartContract, Transaction } from "@thirdweb-dev/sdk";
import type { BigNumber, BigNumberish, BytesLike, ContractInterface, Signer, providers } from "ethers";
import type { WalletConnectReceiverConfig } from "../../../core/types/walletConnect";
import { EVMWallet } from "../../interfaces";
import { WalletOptions } from "../../wallets/base";
import { UserOperationStruct } from "@account-abstraction/contracts";
export type SmartWalletConfig = {
    chain: ChainOrRpcUrl;
    factoryAddress?: string;
    clientId?: string;
    secretKey?: string;
    gasless: boolean;
    bundlerUrl?: string;
    paymasterUrl?: string;
    paymasterAPI?: PaymasterAPI;
    entryPointAddress?: string;
    erc20PaymasterAddress?: string;
    erc20TokenAddress?: string;
} & ContractInfoInput & WalletConnectReceiverConfig;
export type SmartWalletConnectionArgs = {
    personalWallet: EVMWallet;
    accountAddress?: string;
};
export type SmartWalletOptions = WalletOptions;
export interface ProviderConfig extends ContractInfo {
    chain: ChainOrRpcUrl;
    localSigner: Signer;
    entryPointAddress: string;
    clientId?: string;
    secretKey?: string;
    bundlerUrl: string;
    factoryAddress: string;
    accountAddress?: string;
    paymasterAPI: PaymasterAPI;
    gasless: boolean;
    erc20PaymasterAddress?: string;
    erc20TokenAddress?: string;
}
export type ContractInfoInput = {
    factoryInfo?: FactoryContractInfo;
    accountInfo?: AccountContractInfo;
};
export type ContractInfo = {
    factoryInfo: FactoryContractInfoInternal;
    accountInfo: AccountContractInfoInternal;
};
/**
 * @internal
 */
export type AccountContractInfoInternal = {
    abi?: ContractInterface;
    getNonce: (account: SmartContract) => Promise<BigNumber>;
    execute: (account: SmartContract, target: string, value: BigNumberish, data: string) => Promise<Transaction>;
};
/**
 * @internal
 */
export type FactoryContractInfoInternal = {
    abi?: ContractInterface;
    createAccount: (factory: SmartContract, owner: string) => Promise<Transaction>;
    getAccountAddress: (factory: SmartContract, owner: string) => Promise<string>;
};
export type AccountContractInfo = {
    abi?: ContractInterface;
    getNonce?: (account: SmartContract) => Promise<BigNumber>;
    execute?: (account: SmartContract, target: string, value: BigNumberish, data: string) => Promise<Transaction>;
};
export type FactoryContractInfo = {
    abi?: ContractInterface;
    createAccount?: (factory: SmartContract, owner: string) => Promise<Transaction>;
    getAccountAddress?: (factory: SmartContract, owner: string) => Promise<string>;
};
export type PaymasterResult = {
    paymasterAndData: string;
    preVerificationGas?: BigNumber;
    verificationGasLimit?: BigNumber;
    callGasLimit?: BigNumber;
};
/**
 * an API to external a UserOperation with paymaster info
 */
export declare abstract class PaymasterAPI {
    /**
     * @param userOp - a partially-filled UserOperation (without signature and paymasterAndData
     *  note that the "preVerificationGas" is incomplete: it can't account for the
     *  paymasterAndData value, which will only be returned by this method..
     * @returns the value to put into the PaymasterAndData, undefined to leave it empty
     */
    abstract getPaymasterAndData(userOp: Partial<UserOperationStruct>): Promise<PaymasterResult>;
}
export interface TransactionOptions {
    gasless?: boolean;
}
export interface UserOpOptions extends TransactionOptions {
    batchData?: BatchData;
}
export type BatchData = {
    targets: (string | undefined)[];
    data: BytesLike[];
    values: BigNumberish[];
};
export interface BaseApiParams {
    provider: providers.Provider;
    entryPointAddress: string;
    paymasterAPI: PaymasterAPI;
    accountAddress?: string;
    gasless: boolean;
    erc20PaymasterAddress?: string;
    erc20TokenAddress?: string;
}
export interface UserOpResult {
    transactionHash: string;
    success: boolean;
}
export type EstimateUserOpGasRawResult = {
    preVerificationGas: string;
    verificationGas: string;
    verificationGasLimit: string;
    callGasLimit: string;
};
export type EstimateUserOpGasResult = {
    preVerificationGas: BigNumber;
    verificationGas: BigNumber;
    verificationGasLimit: BigNumber;
    callGasLimit: BigNumber;
};
export type ZkTransactionInput = {
    nonce?: string;
    from: string;
    to: string;
    gas: string;
    gasPrice?: string;
    value?: string;
    data?: string;
    maxFeePerGas: string;
    maxPriorityFeePerGas: string;
    chainId?: string;
    signedTransaction: string;
    paymaster: string;
};
//# sourceMappingURL=types.d.ts.map